require_relative 'spec_helper'

describe DbAccessor::Deleter do

  describe ".delete_all" do
    nb = rand(1000)
    api_key = UUID.new.generate
    let(:app) { {model_name: 'Application',content: {name: 'desert'+nb.to_s,api_key: api_key,url: 'http://www.desert'+nb.to_s+'.com',user_id: 2}} }
  let(:user) { {model_name: 'User',content: {login: 'diakhoumpa'+nb.to_s,password: 'passeraa',firstname: 'Ndeye Fatou', lastname: 'Dieng',email: 'diakhouumpa'+nb.to_s+'@gmail.com',gender: 'M'}} }
    context "when the object is deleted" do
      before(:each) do
        DbAccessor::Writer.write_object(app)
        DbAccessor::Writer.write_object(user)
      end
      it do
        found_id = DbAccessor::Reader.simple_select( { model_name: 'User', action_on: 'first' } )[:message].id
        given_param = { model_name: "User", condition: { id: found_id }}
        expect((DbAccessor::Deleter.delete_all(given_param))[:code]).to eq 200
      end

      it do
        found_id = DbAccessor::Reader.simple_select( { model_name: 'Application', action_on: 'first' } )[:message].id
        expect((DbAccessor::Deleter.delete_all({model_name: "Application",condition: { id: found_id }}))[:code]).to eq 200
      end

      it {expect((DbAccessor::Deleter.delete_all({model_name: "User",condition: {login: 'fabira',password: 'passera'}}))[:code]).to eq 200 }
    end

    context "when the conditon format is not valid" do
      param = {model_name: "User",condition: {ta: 1}}

      it {expect((DbAccessor::Deleter.delete_all(param))[:code]).to eq 400 }

      param[:condition] = "coucou"
      it {expect((DbAccessor::Deleter.delete_all(param))[:code]).to eq 400 }

      param[:condition] = nil
      it {expect((DbAccessor::Deleter.delete_all(param))[:code]).to eq 400 }

      param[:condition] = {login: 1}
      it {expect((DbAccessor::Deleter.delete_all(param))[:code]).to eq 400 }

      param[:model_name] = "Application"
      param[:condition] = {login: 'fabi'}
      it {expect((DbAccessor::Deleter.delete_all(param))[:code]).to eq 400 }
    end

    context "when model name is invalid" do
      it {expect((DbAccessor::Deleter.delete_all({model_name: "A",condition: {}}))[:code]).to eq 400 }
    end

    context "when one of the parameter's keys is invalid" do
      it {expect((DbAccessor::Deleter.delete_all({model: "User",condition: {}}))[:code]).to eq 400 }
    end

    context "when the object which satisfy the given condition is deleted" do
      it {expect((DbAccessor::Deleter.delete_all({model_name: "User",condition: {id: 4}}))[:code]).to eq 200 }

      it {expect((DbAccessor::Deleter.delete_all({model_name: "Application",condition: {id: 4}}))[:code]).to eq 200 }
    end

    context "when one or many parameters are empty or nil" do
      it {expect((DbAccessor::Deleter.delete_all({model_name: nil,condition: {id: 3}}))[:code]).to eq 400 }
    end

  end

  describe '.delete' do
    context "When all objects model are really deleted" do
      it{expect(DbAccessor::Deleter.delete({model_name: "User"}.to_json)).to be_a Fixnum }
    end
  end
end
