require_relative 'spec_helper'


describe DbAccessor::Reader do
  nb = rand(50)

  api_key = UUID.new.generate
  let(:app) { {model_name: 'Application',content: {name: 'desert'+nb.to_s,api_key: api_key,url: 'http://www.desert'+nb.to_s+'.com',user_id: 1}} }
  let(:user) { {model_name: 'User',content: {login: 'diakhoumpa'+nb.to_s,password: 'passeraa',firstname: 'Ndeye Fatou', lastname: 'Dieng',email: 'diakhouumpa'+nb.to_s+'@gmail.com',gender: 'M'}} }

  before :each do
    DbAccessor::Writer.write_object(app)
    DbAccessor::Writer.write_object(user)
  end

  describe ".simple_select" do
    context "when all model's items are found" do
      it {expect((DbAccessor::Reader.simple_select({model_name: "User",action_on: "all"}))[:code]).to eq 200 }
      it {expect((DbAccessor::Reader.simple_select({model_name: "Engine",action_on: "all"}))[:code]).to eq 200 }
	  end
    context "when the first model's item is found" do
      it {expect((DbAccessor::Reader.simple_select({model_name: "User",action_on: "first"}))[:code]).to eq 200 }
    end

    context "when the last model's item is found" do
     it {expect((DbAccessor::Reader.simple_select({model_name: "Application",action_on: "last"}))[:code]).to eq 200 }
    end

    context "when it reads many items according to the condition" do
        it {expect((DbAccessor::Reader.simple_select({model_name: "User",action_on: nil}))[:code]).to eq 404 }
    end

    context "when passed parameter's keys are not correct" do
        it {expect((DbAccessor::Reader.simple_select({model: "User",action: nil}))[:code]).to eq 404 }
    end
  end

  describe ".condition_select" do

    context "when it reads one or many items on table with a condition" do
      it do
        found_login = DbAccessor::Reader.simple_select( { model_name: 'User', action_on: 'first' } )[:message].login
        expect((DbAccessor::Reader.condition_select({model_name: "User" , condition: { login: found_login }}))[:code]).to eq 200
      end
	end

    context "when the condition is not satisfied" do
      it {expect((DbAccessor::Reader.condition_select({model_name: "Application" , condition: {name: "fab"}}))[:code]).to eq 404 }
	end

    context "when one or many parameters are empty or nil" do
      it {expect((DbAccessor::Reader.condition_select({model_name: "", condition: nil }))[:code]).to eq 404 }
    end

    context "when condition is not correct" do
      it {expect((DbAccessor::Reader.condition_select({model_name: "", condition: 1 }))[:code]).to eq 404 }

      it {expect((DbAccessor::Reader.condition_select({model_name: "", condition: "pas une condition"}))[:code]).to eq 404 }
    end

    context "when passed parameter's keys are not correct" do
        it {expect((DbAccessor::Reader.simple_select({model: "User",cond: nil}))[:code]).to eq 404 }
    end
  end

  describe ".select_by_id" do

    context "when it reads on a model with an id" do
      it {expect((DbAccessor::Reader.select_by_id({model_name: "User",identifiers: [5]}))[:code]).to eq 200 }

      it {expect((DbAccessor::Reader.select_by_id({model_name: "User",identifiers: [5]}))[:message]).to_not eq nil }
	end

    context "when one of the ids is incorrect" do
      it {expect((DbAccessor::Reader.select_by_id({model_name: "User",identifiers: [1,2,'c']}))[:code]).to eq 404 }
	end

    context "when one of the parameter is nil" do
        it {expect((DbAccessor::Reader.select_by_id({model_name: "",identifiers: [1,2,3]}))[:code]).to eq 404 }
    end

  end

  describe ".linked_models_select" do

    context "when it reads models on model with a condition" do
      it {expect((DbAccessor::Reader.linked_models_select({model_name: "User", condition: { id: 1}, models: "applications"}))[:code]).to eq 200 }
	end

    context "when a parameter is empty or empty" do
        it {expect((DbAccessor::Reader.linked_models_select({model_name: "", condition: nil, models: nil }))[:code]).to eq 404 }
    end

    context "when models are incorrect" do
        it {expect((DbAccessor::Reader.linked_models_select({model_name: "", condition: nil, models: "pas de model" }))[:code]).to eq 404 }
    end

  end


end
