require_relative 'spec_helper'
require 'redis'
require 'uuid'

describe DbAccessor::Updater do

  nb = rand(50)

  describe ".update_by_id" do

	context "updates one or many users by id" do
    DbAccessor::Writer.write_object({model_name: 'User',content: {login: 'diakhoumpa'+nb.to_s,password: 'passeraa',firstname: 'Ndeye Fatou', lastname: 'Dieng',email: 'diakhouumpa'+nb.to_s+'@gmail.com',gender: 'M'}})
    found_id = DbAccessor::Reader.simple_select( { model_name: 'User', action_on: 'first' } )[:message].id
	  param = {model_name: "User",attributes: { found_id => {login: "mamadouPouye"}} }
	  it {expect((DbAccessor::Updater.update_by_id(param))[:code]).to eq 200 }
	end

	context "updates one or many applications by id" do
    api_key = UUID.new.generate
    DbAccessor::Writer.write_object({model_name: 'Application',content: {name: 'desert'+nb.to_s,api_key: api_key,url: 'http://www.desert'+nb.to_s+'.com',user_id: 1}})
    found_id = DbAccessor::Reader.simple_select( { model_name: 'Application', action_on: 'first' } )[:message].id
	  param = {model_name: "Application",attributes: {found_id => {api_key: UUID.new.generate}} }
	  it {expect((DbAccessor::Updater.update_by_id(param))[:code]).to eq 200 }

    DbAccessor::Writer.write_object({model_name: 'Engine',content: {name: 'engine'+nb.to_s,engine_conf: {action: ['LIKE','SHARE'],score: [2,5],algorithm: 'Person',type: 'UserBased', neighborhood: 5 }.to_json ,application_id: 1}})
    found_id = DbAccessor::Reader.simple_select( { model_name: 'Engine', action_on: 'first' } )[:message].id
	  param = {model_name: "Engine",attributes: { found_id => {engine_conf: {similarity_metric: 'similarity_metric',reco_algo: 'reco_algo',neighborhood: 'neighborhood'}.to_json} } }
	  it {expect((DbAccessor::Updater.update_by_id(param))[:code]).to eq 200 }
	end

    context "when attributes format is not correct" do
	  param = {model_name: "Application",attributes: { 'c' => { name: "apache"},2 => {name: "site du zero"}} }
	  it {expect((DbAccessor::Updater.update_by_id(param))[:code]).to eq 400 }

	  param[:attributes] = { 3 => { na: "apache"},2 => {name: "site du zero"}}
	  it {expect((DbAccessor::Updater.update_by_id(param))[:code]).to eq 400 }

	  param[:attributes] = { 3 => { name: 1},2 => {name: "site du zero"}}
	  it {expect((DbAccessor::Updater.update_by_id(param))[:code]).to eq 400 }

	  it {expect((DbAccessor::Updater.update_by_id(param))[:code]).to eq 400 }
    param[:attributes] = { 3 => nil,2 => {name: "site du zero"}}

	  param[:attributes] = { 3 => " ",2 => {name: "site du zero"}}
	  it {expect((DbAccessor::Updater.update_by_id(param))[:code]).to eq 400 }

	  param[:attributes] = { 3 => {},2 => {name: "site du zero"}}
	  it {expect((DbAccessor::Updater.update_by_id(param))[:code]).to eq 400 }

	  param[:attributes] = { 3 => {name: nil},2 => {name: "site du zero"}}
	  it {expect((DbAccessor::Updater.update_by_id(param))[:code]).to eq 400 }

	  param[:attributes] = { 3 => {na: 'to'},2 => {name: "site du zero"}}
	  it {expect((DbAccessor::Updater.update_by_id(param))[:code]).to eq 400 }
	end

	context "when the object is not found" do
	  param = {model_name: "User",attributes: { 10000 => { login: "fabi"}} }
      it {expect((DbAccessor::Updater.update_by_id(param))[:code]).to eq 400 }
    end

    context "when the passed parameter is nil" do
      it {expect((DbAccessor::Updater.update_by_id(nil))[:code]).to eq 400 }
    end

    context "when attributes to update are empty" do
      it {expect((DbAccessor::Updater.update_by_id(model_name: "User",attributes: {}))[:code]).to eq 400 }
    end

    context "when the model name is incorrect" do
      it {expect((DbAccessor::Updater.update_by_id(model_name: "Us",attributes: {}))[:code]).to eq 400 }
    end

    context "when the attributes are not correct" do
      it {expect((DbAccessor::Updater.update_by_id(model_name: "Application",attributes: "coucou"))[:code]).to eq 400 }
    end

  end



   describe ".update_all_objects" do
	parameter={model_name:  "User", attributes: {password: "passe#{nb}"}}

  context "when it updates all model objects" do
      it {expect((DbAccessor::Updater.update_all_objects(parameter))[:code]).to eq 200 }
  end

    context "when all model objects that satisfy the condition are updated" do
	  it do
      Redis.current.del('users')
      parameter={model_name:  "User", attributes: {password: "password#{nb}"}}
      found_login = DbAccessor::Reader.simple_select( { model_name: 'User', action_on: 'last' } )[:message].login
      condition = { login: found_login }
      expect((DbAccessor::Updater.update_all_objects(parameter,condition))[:code]).to eq 200
    end
	end

    context "when model's attributes are not correct" do
      param={model_name:  "User", attributes: {login: 'fabira',password: 3}}
      it {expect((DbAccessor::Updater.update_all_objects(param))[:code]).to eq 400 }
    end

    context "when one of the model's attributes is nil" do
      param={model_name:  "User", attributes: {login: 'fabira',password: nil}}
      it {expect((DbAccessor::Updater.update_all_objects(param))[:code]).to eq 400 }
    end

    context "when attributes'keys are not correct" do
      param={model_name:  "User", attributes: {lon: 'fabira',pasord: 'pass'}}
      it {expect((DbAccessor::Updater.update_all_objects(param))[:code]).to eq 400 }
    end

    context "returns false when attributes keys don't correspond to the model" do
      param={model_name:  "Application", attributes: {login: 'fabira3',password: 'passera'}}
      it {expect((DbAccessor::Updater.update_all_objects(param))[:code]).to eq 400 }
    end
  end
end
