require_relative 'spec_helper'


describe DbAccessor::Writer do

  nb      = rand(1000)
  api_key = UUID.new.generate

  let(:user) { {model_name: 'User',content: {login: 'diakhoumpa'+nb.to_s,password: 'passeraa',firstname: 'Ndeye Fatou', lastname: 'Dieng',email: 'diakhouumpa'+nb.to_s+'@gmail.com',gender: 'M'}} }

  let(:app) { {model_name: 'Application',content: {name: 'desert'+nb.to_s,api_key: api_key,url: 'http://www.desert'+nb.to_s+'.com',user_id: 2}} }

  let(:engine) {  {model_name: 'Engine',content: {name: 'engine'+nb.to_s,engine_conf: {action: ['LIKE','SHARE'],score: [2,5],algorithm: 'Person',type: 'UserBased', neighborhood: 5 }.to_json ,application_id: 1}} }

    context "when a user is saved in database" do
      it {expect((DbAccessor::Writer.write_object(user))[:code]).to eq 201}
    end

    context "when an admin user is saved in database" do
      it do
        admin_user = user
        admin_user[:content][:login] = "admin#{nb.to_s}"
        admin_user[:content][:email] = "admin#{nb.to_s}@gmail.com"
        admin_user[:content][:admin] = true
        expect((DbAccessor::Writer.write_object(admin_user))[:code]).to eq 201
      end
    end

    context "when an application is saved in database" do
      it {expect((DbAccessor::Writer.write_object(app))[:code]).to eq 201}
    end

    context "when an engine is saved in database" do
      it {expect((DbAccessor::Writer.write_object(engine))[:code]).to eq 201}
    end

    context "when the object's parameters are empty" do
     it {expect((DbAccessor::Writer.write_object({model_name: '',content: user[:content]}))[:code]).to eq 400 }
    end

    context "when the model_name contains numbers" do
     it {expect((DbAccessor::Writer.write_object({model_name: 'DGH67889',content: user[:content]}))[:code]).to eq 400 }
    end

    context "when passed parameter's keys are incorrect" do
     it {expect((DbAccessor::Writer.write_object({model: 'DGH67889',con: user[:content]}))[:code]).to eq 400 }
    end

    context "when the parameter is not a hash" do
      it { expect((DbAccessor::Writer.write_object("coucou"))[:code]).to eq 400 }
    end

    context "when the parameter's keys are not correct" do
      it { expect((DbAccessor::Writer.write_object({t: "coucou"}))[:code]).to eq 400 }
    end

    context "when required parameters are not given" do
     user =  {model_name: 'User',content: {firstname: 'ablaye', lastname: 'Dieng',gender: 'M'} }
     it { expect((DbAccessor::Writer.write_object(user))[:code]).to eq 400 }
    end

    context "when required parameters are not correct" do
     user = {model_name: 'User',content: {login: nb.to_s,password: 'passer',firstname: 'Ndeye Fatou', lastname: 'Dieng',email: 'fatou',gender: 'M'}}
     it { expect((DbAccessor::Writer.write_object(user))[:code]).to eq 400 }
    end

    context "when the content doesn't correspond to the model_name" do
     it { expect((DbAccessor::Writer.write_object( {model_name: 'User',content: {name: 'Wiki',api_key: api_key,url: 'http://www.wiki.com',user_id: 2}} ))[:code]).to eq 400 }
    end

    context "when the content is incorrect" do
     it { expect((DbAccessor::Writer.write_object( {model_name: 'Utilisateur',content: "coucou"} ))[:code]).to eq 400 }
    end

    context "when the model_name is missing" do
     it { expect((DbAccessor::Writer.write_object( {model_name: '',content: {name: 'Wiki',api_key: api_key,url: 'http://www.wiki.com',user_id: 2}} ))[:code]).to eq 400 }
    end

    context "when the content is not correct" do
     it { expect((DbAccessor::Writer.write_object( {model_name: 'Application',content: {login: nb.to_s,password: 'passer',firstname: 'Ndeye Fatou', lastname: 'Dieng',email: 'fatou',gender: 'M'}} ))[:code]).to eq 400 }
    end

  end
